// copied from the dUFLP for use here

/*
	FileName: 		SaveEnv.cc

   Author:			Gary White

   Description:	Saves VdB environment and allows restoring all,
   					or individual settings.  This is useful during
                  application development so that you can save
                  the development environment during application
                  initialization and easily restore it when the
                  application quits.  Can also be useful for 
                  debugging purposes by saving the environment
                  settings at a point in your code where you are
                  having a problem and calling the WriteToFile 
                  method to save the settings at that point.

   Version History:

   					May 7, 2000	-	Original version

   Usage:			// Automatically saves the environment when
   					// the object is created

                  set proc to SaveEnv.cc additive
   					_app.OldEnv = new SaveEnv()

                  // restore individual value
                  _app.OldEnv.restore("cuaenter")

                  // restore entire environment
                  _app.OldEnv.restore()

   Caveat:			Those settings that require the word TO when
   					setting them, must have the TO included with
                  the setting name when restoring an individual
                  environment setting, i.e.:

                  _app.OldEnv.restore("decimals to")

                  This is necessary because both BELL and CURRENCY
                  may be set either way with a different result 
                  depending upon whether the TO is used:

                  SET BELL ON or SET BELL OFF
                  SET BELL TO <freq>, <duration>

                  SET CURRENCY LEFT or SET CURRENCY RIGHT
                  SET CURRENCY TO "$"

                  Without including the TO keyword, there would
                  be no way of knowing which setting to restore.
                  See the Init() method for the correct setting 
                  names.

License:

   This object is FREEWARE. You are free to include this
   object in your own Visual dBASE programs and distribute it
   freely as part of your programs as object code.

   You may freely distribute this object to other developers
   for inclusion in their programs.

   You may NOT sell this object or use it in any way whereby
   it is NOT included in a program.

   You may make any revisions desired to this code.

   Have fun with it!

      Gary

******************************************************************
*                                                                *
*  Warning: This code is not warranteed in any way whatsoever.   *
*  Use it at your own risk. Your use of this code releases Gary  *
*  White from any liability whatsoever as regards the use,       *
*  suitability or quality of this object.                        *
*                                                                *
******************************************************************
   
*/


class SaveEnv

	this.settings = new assocArray()
	this.init()

   function init
      with (this)
         settings['ALTERNATE'] = set('ALTERNATE')
         settings['AUTOSAVE'] = set('AUTOSAVE')
         settings['BELL'] = set('BELL')
         settings['BELL TO'] = setto('BELL')
         settings['BLOCKSIZE TO'] = set('BLOCKSIZE')
         settings['CENTURY'] = set('CENTURY')
         settings['CONFIRM'] = set('CONFIRM')
         settings['CONSOLE'] = set('CONSOLE')
         settings['COVERAGE'] = set('COVERAGE')
         settings['CUAENTER'] = set('CUAENTER')
         settings['CURRENCY'] = set('CURRENCY')
         settings['CURRENCY TO'] = ["]+setto('CURRENCY')+["]
         settings['DATABASE TO'] = set('DATABASE')
         settings['DATE'] = set('DATE')
         settings['DBTYPE TO'] = set('DBTYPE')
         settings['DECIMALS TO'] = set('DECIMALS')
         settings['DELETED'] = set('DELETED')
         settings['DESIGN'] = set('DESIGN')
         settings['DEVELOPMENT'] = set('DEVELOPMENT')
         settings['DIRECTORY TO'] = ["]+set('DIRECTORY')+["]
         settings['EDITOR TO'] = set('EDITOR')
         settings['ENCRYPTION'] = set('ENCRYPTION')
         settings['EPOCH TO'] = set('EPOCH')
         settings['ERROR TO'] = set('ERROR')
         settings['ESCAPE'] = set('ESCAPE')
         settings['EXACT'] = set('EXACT')
         settings['EXCLUSIVE'] = set('EXCLUSIVE')
         settings['FIELDS TO'] = set('FIELDS')
         settings['FULLPATH'] = set('FULLPATH')
         settings['HEADINGS'] = set('HEADINGS')
         settings['IBLOCK TO'] = set('IBLOCK')
         settings['LDCHECK'] = set('LDCHECK')
         settings['LDCONVERT'] = set('LDCONVERT')
         settings['LOCK'] = set('LOCK')
         settings['MARGIN TO'] = set('MARGIN')
         settings['MARK TO'] = ["]+set('MARK')+["]
         settings['MBLOCK TO'] = set('MBLOCK')
         settings['MESSAGE TO'] = set('MESSAGE')
         settings['NEAR'] = set('NEAR')
         settings['ODOMETER TO'] = set('ODOMETER')
         settings['PATH TO'] = iif(empty(set('PATH')),"",["]+set('PATH')+["])
         settings['POINT TO'] = ["]+set('POINT')+["]
         settings['PRECISION TO'] = set('PRECISION')
         settings['REFRESH TO'] = set('REFRESH')
         settings['REPROCESS TO'] = set('REPROCESS')
         settings['SAFETY'] = set('SAFETY')
         settings['SEPARATOR TO'] = ["]+set('SEPARATOR')+["]
         settings['SPACE'] = set('SPACE')
         settings['TALK'] = set('TALK')
         settings['TITLE'] = set('TITLE')
         settings['TYPEAHEAD TO'] = set('TYPEAHEAD')
         settings['UNIQUE'] = set('UNIQUE')
         settings['SPEEDBAR'] = _app.speedbar
         settings['STATUSBAR'] = _app.statusbar
      endwith
      this.oldText = _app.framewin.text
   	return

   function release
   	release object this
      return

   function restore( cRestore )
   	private cCmd, cSetting	// private so we can &macro and type() them
      if empty( cRestore )
         cSetting = this.settings.firstkey
         if not empty(cSetting)
            do
               if type('this.settings[cSetting]') == "L"
                  cCmd = '_app.'+cSetting+'='+this.settings[cSetting]
               else
                  cCmd = 'set ' + cSetting + ' ' + this.settings[cSetting]
               endif
               try
                  &cCmd.
               catch(exception e)
               endtry
               cSetting = this.settings.nextKey(cSetting)
            until empty(cSetting)
         endif
         release cCmd, cSetting
      else
      	if this.settings.isKey( upper(cRestore) )
         	cSetting = upper(cRestore)
            if type('this.settings[cSetting]') == "L"
               cCmd = '_app.'+cSetting+'='+this.settings[cSetting]
            else
               cCmd = 'set ' + cSetting + ' ' + this.settings[cSetting]
            endif
            try
               &cCmd.
            catch(exception e)
            endtry
         else
         	MsgBox('Setting: '+cRestore+' not found','Error',16)
         endif
      endif
      _app.framewin.text := this.oldText
   	return

   function WriteToFile( cFileName )
   	// saves settings to a file
      // NOTE:  Will overwrite without warning regardless
      //        of SET SAFETY
   	local bRet, f
      private cSetting	// so we can TYPE() it
      bRet = true
      if not empty( cFileName )
         try
            f = new file()
            f.create( cFileName, "w" )
            f.writeln("Environment settings - " + datetime())
            f.writeln("------------------------------------------")
            cSetting = this.settings.firstKey
            do
               if type('this.settings[cSetting]') == "L"
                  cCmd = '_app.'+cSetting+'='+this.settings[cSetting]
               else
                  cCmd = 'SET ' + cSetting + ' ' + this.settings[cSetting]
               endif
               f.writeln( cCmd )
               cSetting := this.settings.nextKey( cSetting )
            until empty( cSetting )
         catch(exception e)
            msgbox(e.message, "Error: "+e.code, 16)
         finally
            f.close()
         endtry
      else
      	msgbox("File name missing", "Save environment error", 16)
      endif
   	return bRet

endclass

